/**
 * @file
 * Background rotate javascript.
 */

(function ($) {
  'use strict';
  Drupal.behaviors.bg_rotate_admin = {

  };

  Drupal.behaviors.bg_rotate = {
    // Emulate PHP's srand function with date.
    dsRand: function (nseed) {
      var seed;
      var constant = Math.pow(2, 13) + 1;
      var prime = 37;
      var maximum = Math.pow(2, 50);

      // Set seed if argument is passed.
      if (nseed) {
        seed = nseed;
      }

      // If there is no seed, use current timestamp.
      if (seed === null) {
        seed = (new Date()).getTime();
      }

      seed *= constant;
      seed += prime;
      seed %= maximum;

      return seed;
    },
    randImage: function (urls, interval) {
      // "Extend" Date prototype to enable week number.
      // @todo Wait until EC6 is standard and properly extend it instead.
      Date.prototype.getWeek = function () {
        var oneJan = new Date(this.getFullYear(), 0, 1);
        var week = Math.ceil((((this - oneJan) / 86400000) + oneJan.getDay() + 1) / 7);
        return week;
      };

      // Create date object.
      var date = new Date();

      // We want to create a seed based on an interval
      // that we specify in settings.
      var seed;
      switch (interval) {
        case 'month': seed = date.getMonth();
          break;

        case 'week': seed = date.getWeek();
          break;

        case 'day': seed = date.getDate();
          break;

        case 'hour': seed = date.getHours();
          break;

        case 'minute': seed = date.getMinutes();
          break;

        case 'second': seed = date.getSeconds();
          break;

      }

      // Get array length.
      var arrayCount = urls.length;

      // Generate a random number by current seed.
      // This will change based on the interval set.
      var randNumber = this.dsRand(seed);

      // Get modulus to get a valid array key.
      var arrayKey = randNumber % arrayCount;

      // Fetch image url object from settings array.
      var urlObject = urls[arrayKey];

      // Get page width.
      var goal = window.innerWidth;
      // Out breakpoints.
      var widths = [1200, 930, 650];

      // Get closest breakpoint to width.
      var closest = widths.reduce(function (prev, curr) {
        return (Math.abs(curr - goal) < Math.abs(prev - goal) ? curr : prev);
      });

      // Assign images to breakpoints.
      var images = {
        1200: urlObject.screen,
        930: urlObject.tablet,
        650: urlObject.phone
      };

      // Return closest random image size.
      return images[closest];
    },

    attach: function (context, settings) {
      var self = this;
      // Check if image rotate settings is present or abort.
      if (typeof settings.bg_rotate === 'undefined') {
        return;
      }

      // Get our configuration.
      var conf = settings.bg_rotate;

      // Get image based on window width and image styles.
      var image = self.randImage(conf.urls, conf.interval);

      // Check if there is an image.
      if (typeof image !== 'undefined') {
        // CSS properties.
        var css = {
          'background-image': 'url(' + image + ')',
          'background-repeat': conf.css.repeat,
          'background-position': conf.css.position,
          'background-attachment': conf.css.attachment,
          'background-size': conf.css.size
        };

        // Attach random image.
        $(conf.selector)
          .css(css)
          .addClass('bg_rotate-element');
      }
    }
  };
})(jQuery);
